import { TSESLint } from '@typescript-eslint/utils';

interface RxjsXRuleDocs {
    description: string;
    recommended?: TSESLint.RuleRecommendation | TSESLint.RuleRecommendationAcrossConfigs<unknown[]>;
    requiresTypeChecking?: boolean;
}

interface ChecksVoidReturnOptions {
    arguments?: boolean;
    attributes?: boolean;
    inheritedMethods?: boolean;
    properties?: boolean;
    returns?: boolean;
    variables?: boolean;
}

declare const rxjsX: {
    configs: {
        recommended: {
            name: "rxjs-x/recommended";
            plugins: {
                'rxjs-x': TSESLint.FlatConfig.Plugin;
            };
            rules: {
                'rxjs-x/no-async-subscribe': "error";
                'rxjs-x/no-create': "error";
                'rxjs-x/no-ignored-notifier': "error";
                'rxjs-x/no-ignored-replay-buffer': "error";
                'rxjs-x/no-ignored-takewhile-value': "error";
                'rxjs-x/no-implicit-any-catch': "error";
                'rxjs-x/no-index': "error";
                'rxjs-x/no-internal': "error";
                'rxjs-x/no-nested-subscribe': "error";
                'rxjs-x/no-redundant-notify': "error";
                'rxjs-x/no-sharereplay': "error";
                'rxjs-x/no-subject-unsubscribe': "error";
                'rxjs-x/no-subscribe-in-pipe': "error";
                'rxjs-x/no-topromise': "error";
                'rxjs-x/no-unbound-methods': "error";
                'rxjs-x/no-unsafe-subject-next': "error";
                'rxjs-x/no-unsafe-takeuntil': "error";
                'rxjs-x/prefer-observer': "error";
                'rxjs-x/prefer-root-operators': "error";
                'rxjs-x/throw-error': "error";
            };
        };
        strict: {
            name: "rxjs-x/strict";
            plugins: {
                'rxjs-x': TSESLint.FlatConfig.Plugin;
            };
            rules: {
                'rxjs-x/no-async-subscribe': "error";
                'rxjs-x/no-create': "error";
                'rxjs-x/no-exposed-subjects': "error";
                'rxjs-x/no-floating-observables': "error";
                'rxjs-x/no-ignored-default-value': "error";
                'rxjs-x/no-ignored-error': "error";
                'rxjs-x/no-ignored-notifier': "error";
                'rxjs-x/no-ignored-replay-buffer': "error";
                'rxjs-x/no-ignored-takewhile-value': "error";
                'rxjs-x/no-implicit-any-catch': ["error", {
                    allowExplicitAny: false;
                }];
                'rxjs-x/no-index': "error";
                'rxjs-x/no-internal': "error";
                'rxjs-x/no-misused-observables': "error";
                'rxjs-x/no-nested-subscribe': "error";
                'rxjs-x/no-redundant-notify': "error";
                'rxjs-x/no-sharereplay': "error";
                'rxjs-x/no-subclass': "error";
                'rxjs-x/no-subject-unsubscribe': "error";
                'rxjs-x/no-subscribe-in-pipe': "error";
                'rxjs-x/no-topromise': "error";
                'rxjs-x/no-unbound-methods': "error";
                'rxjs-x/no-unsafe-subject-next': "error";
                'rxjs-x/no-unsafe-takeuntil': "error";
                'rxjs-x/prefer-observer': "error";
                'rxjs-x/prefer-root-operators': "error";
                'rxjs-x/throw-error': ["error", {
                    allowThrowingAny: false;
                    allowThrowingUnknown: false;
                }];
            };
        };
    };
    meta: {
        name: string;
        version: string;
    };
    rules: {
        'ban-observables': TSESLint.RuleModule<"forbidden", readonly Record<string, string | boolean>[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'ban-operators': TSESLint.RuleModule<"forbidden", readonly Record<string, string | boolean>[], RxjsXRuleDocs, TSESLint.RuleListener>;
        finnish: TSESLint.RuleModule<"shouldBeFinnish" | "shouldNotBeFinnish", readonly {
            functions?: boolean;
            methods?: boolean;
            names?: Record<string, boolean>;
            parameters?: boolean;
            properties?: boolean;
            strict?: boolean;
            types?: Record<string, boolean>;
            variables?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        just: TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        macro: TSESLint.RuleModule<"macro", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-async-subscribe': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-compat': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-connectable': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-create': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-cyclic-action': TSESLint.RuleModule<"forbidden", readonly {
            observable?: string;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-explicit-generics': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-exposed-subjects': TSESLint.RuleModule<"forbidden" | "forbiddenAllowProtected", readonly {
            allowProtected?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-finnish': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-floating-observables': TSESLint.RuleModule<"forbidden" | "forbiddenNoVoid", readonly {
            ignoreVoid?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-ignored-default-value': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-ignored-error': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-ignored-notifier': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-ignored-replay-buffer': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-ignored-subscribe': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-ignored-subscription': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-ignored-takewhile-value': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-implicit-any-catch': TSESLint.RuleModule<"explicitAny" | "implicitAny" | "narrowed" | "suggestExplicitUnknown", readonly {
            allowExplicitAny?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-index': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-internal': TSESLint.RuleModule<"forbidden" | "suggest", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-misused-observables': TSESLint.RuleModule<"forbiddenVoidReturnArgument" | "forbiddenVoidReturnAttribute" | "forbiddenVoidReturnInheritedMethod" | "forbiddenVoidReturnProperty" | "forbiddenVoidReturnReturnValue" | "forbiddenVoidReturnVariable" | "forbiddenSpread", readonly {
            checksVoidReturn?: boolean | ChecksVoidReturnOptions;
            checksSpreads?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-nested-subscribe': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-redundant-notify': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-sharereplay': TSESLint.RuleModule<"forbidden" | "forbiddenWithoutConfig", readonly {
            allowConfig?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-subclass': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-subject-unsubscribe': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-subject-value': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-subscribe-handlers': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-subscribe-in-pipe': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-tap': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-topromise': TSESLint.RuleModule<"forbidden" | "suggestLastValueFromWithDefault" | "suggestLastValueFrom" | "suggestFirstValueFrom", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-unbound-methods': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-unsafe-catch': TSESLint.RuleModule<"forbidden", readonly {
            observable?: string;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-unsafe-first': TSESLint.RuleModule<"forbidden", readonly {
            observable?: string;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-unsafe-subject-next': TSESLint.RuleModule<"forbidden", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-unsafe-switchmap': TSESLint.RuleModule<"forbidden", readonly {
            allow?: string | string[];
            disallow?: string | string[];
            observable?: string;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'no-unsafe-takeuntil': TSESLint.RuleModule<"forbidden", readonly {
            alias?: string[];
            allow?: string[];
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'prefer-observer': TSESLint.RuleModule<"forbidden", readonly {
            allowNext?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'prefer-root-operators': TSESLint.RuleModule<"forbidden" | "suggest" | "forbiddenWithoutFix", [], RxjsXRuleDocs, TSESLint.RuleListener>;
        'suffix-subjects': TSESLint.RuleModule<"forbidden", readonly {
            parameters?: boolean;
            properties?: boolean;
            suffix?: string;
            types?: Record<string, boolean>;
            variables?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
        'throw-error': TSESLint.RuleModule<"forbidden", readonly {
            allowThrowingAny?: boolean;
            allowThrowingUnknown?: boolean;
        }[], RxjsXRuleDocs, TSESLint.RuleListener>;
    };
};

export { rxjsX as default };
